#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 8
#endif
#ifndef PFXDIST
   #define PFXDIST 0
#endif
#ifndef PFADIST
   #define PFADIST 0
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
void ATL_UGEMV(ATL_CINT M, ATL_CINT N, const TYPE *A, ATL_CINT lda,
               const TYPE *X, TYPE *Y)
/*
 * 128x8 unrolled mvt_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N8=(N/8)*8, M128=(M/128)*128, lda8=lda*8;
   ATL_INT j;

   for (j=N8; j; j -= 8, A += lda8, Y += 8)
   {
      const float *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda, *A4=A3+lda,
                  *A5=A4+lda, *A6=A5+lda, *A7=A6+lda;
      register float y0=ATL_rzero, y1=ATL_rzero, y2=ATL_rzero, y3=ATL_rzero,
                     y4=ATL_rzero, y5=ATL_rzero, y6=ATL_rzero, y7=ATL_rzero;
      ATL_INT i;
      prefY(Y+8+8-1);
      for (i=0; i < M128; i += 128)
      {
         const register float x0=X[i+0], x1=X[i+1], x2=X[i+2], x3=X[i+3],
                              x4=X[i+4], x5=X[i+5], x6=X[i+6], x7=X[i+7],
                              x8=X[i+8], x9=X[i+9], x10=X[i+10], x11=X[i+11],
                              x12=X[i+12], x13=X[i+13], x14=X[i+14],
                              x15=X[i+15], x16=X[i+16], x17=X[i+17],
                              x18=X[i+18], x19=X[i+19], x20=X[i+20],
                              x21=X[i+21], x22=X[i+22], x23=X[i+23],
                              x24=X[i+24], x25=X[i+25], x26=X[i+26],
                              x27=X[i+27], x28=X[i+28], x29=X[i+29],
                              x30=X[i+30], x31=X[i+31], x32=X[i+32],
                              x33=X[i+33], x34=X[i+34], x35=X[i+35],
                              x36=X[i+36], x37=X[i+37], x38=X[i+38],
                              x39=X[i+39], x40=X[i+40], x41=X[i+41],
                              x42=X[i+42], x43=X[i+43], x44=X[i+44],
                              x45=X[i+45], x46=X[i+46], x47=X[i+47],
                              x48=X[i+48], x49=X[i+49], x50=X[i+50],
                              x51=X[i+51], x52=X[i+52], x53=X[i+53],
                              x54=X[i+54], x55=X[i+55], x56=X[i+56],
                              x57=X[i+57], x58=X[i+58], x59=X[i+59],
                              x60=X[i+60], x61=X[i+61], x62=X[i+62],
                              x63=X[i+63], x64=X[i+64], x65=X[i+65],
                              x66=X[i+66], x67=X[i+67], x68=X[i+68],
                              x69=X[i+69], x70=X[i+70], x71=X[i+71],
                              x72=X[i+72], x73=X[i+73], x74=X[i+74],
                              x75=X[i+75], x76=X[i+76], x77=X[i+77],
                              x78=X[i+78], x79=X[i+79], x80=X[i+80],
                              x81=X[i+81], x82=X[i+82], x83=X[i+83],
                              x84=X[i+84], x85=X[i+85], x86=X[i+86],
                              x87=X[i+87], x88=X[i+88], x89=X[i+89],
                              x90=X[i+90], x91=X[i+91], x92=X[i+92],
                              x93=X[i+93], x94=X[i+94], x95=X[i+95],
                              x96=X[i+96], x97=X[i+97], x98=X[i+98],
                              x99=X[i+99], x100=X[i+100], x101=X[i+101],
                              x102=X[i+102], x103=X[i+103], x104=X[i+104],
                              x105=X[i+105], x106=X[i+106], x107=X[i+107],
                              x108=X[i+108], x109=X[i+109], x110=X[i+110],
                              x111=X[i+111], x112=X[i+112], x113=X[i+113],
                              x114=X[i+114], x115=X[i+115], x116=X[i+116],
                              x117=X[i+117], x118=X[i+118], x119=X[i+119],
                              x120=X[i+120], x121=X[i+121], x122=X[i+122],
                              x123=X[i+123], x124=X[i+124], x125=X[i+125],
                              x126=X[i+126], x127=X[i+127];

         prefX(X);
         y0 += A0[i+0] * x0;
         prefA(A0);
         y1 += A1[i+0] * x0;
         prefA(A1);
         y2 += A2[i+0] * x0;
         prefA(A2);
         y3 += A3[i+0] * x0;
         prefA(A3);
         y4 += A4[i+0] * x0;
         prefA(A4);
         y5 += A5[i+0] * x0;
         prefA(A5);
         y6 += A6[i+0] * x0;
         prefA(A6);
         y7 += A7[i+0] * x0;
         prefA(A7);
         y0 += A0[i+1] * x1;
         y1 += A1[i+1] * x1;
         y2 += A2[i+1] * x1;
         y3 += A3[i+1] * x1;
         y4 += A4[i+1] * x1;
         y5 += A5[i+1] * x1;
         y6 += A6[i+1] * x1;
         y7 += A7[i+1] * x1;
         y0 += A0[i+2] * x2;
         y1 += A1[i+2] * x2;
         y2 += A2[i+2] * x2;
         y3 += A3[i+2] * x2;
         y4 += A4[i+2] * x2;
         y5 += A5[i+2] * x2;
         y6 += A6[i+2] * x2;
         y7 += A7[i+2] * x2;
         y0 += A0[i+3] * x3;
         y1 += A1[i+3] * x3;
         y2 += A2[i+3] * x3;
         y3 += A3[i+3] * x3;
         y4 += A4[i+3] * x3;
         y5 += A5[i+3] * x3;
         y6 += A6[i+3] * x3;
         y7 += A7[i+3] * x3;
         y0 += A0[i+4] * x4;
         y1 += A1[i+4] * x4;
         y2 += A2[i+4] * x4;
         y3 += A3[i+4] * x4;
         y4 += A4[i+4] * x4;
         y5 += A5[i+4] * x4;
         y6 += A6[i+4] * x4;
         y7 += A7[i+4] * x4;
         y0 += A0[i+5] * x5;
         y1 += A1[i+5] * x5;
         y2 += A2[i+5] * x5;
         y3 += A3[i+5] * x5;
         y4 += A4[i+5] * x5;
         y5 += A5[i+5] * x5;
         y6 += A6[i+5] * x5;
         y7 += A7[i+5] * x5;
         y0 += A0[i+6] * x6;
         y1 += A1[i+6] * x6;
         y2 += A2[i+6] * x6;
         y3 += A3[i+6] * x6;
         y4 += A4[i+6] * x6;
         y5 += A5[i+6] * x6;
         y6 += A6[i+6] * x6;
         y7 += A7[i+6] * x6;
         y0 += A0[i+7] * x7;
         y1 += A1[i+7] * x7;
         y2 += A2[i+7] * x7;
         y3 += A3[i+7] * x7;
         y4 += A4[i+7] * x7;
         y5 += A5[i+7] * x7;
         y6 += A6[i+7] * x7;
         y7 += A7[i+7] * x7;
         y0 += A0[i+8] * x8;
         y1 += A1[i+8] * x8;
         y2 += A2[i+8] * x8;
         y3 += A3[i+8] * x8;
         y4 += A4[i+8] * x8;
         y5 += A5[i+8] * x8;
         y6 += A6[i+8] * x8;
         y7 += A7[i+8] * x8;
         y0 += A0[i+9] * x9;
         y1 += A1[i+9] * x9;
         y2 += A2[i+9] * x9;
         y3 += A3[i+9] * x9;
         y4 += A4[i+9] * x9;
         y5 += A5[i+9] * x9;
         y6 += A6[i+9] * x9;
         y7 += A7[i+9] * x9;
         y0 += A0[i+10] * x10;
         y1 += A1[i+10] * x10;
         y2 += A2[i+10] * x10;
         y3 += A3[i+10] * x10;
         y4 += A4[i+10] * x10;
         y5 += A5[i+10] * x10;
         y6 += A6[i+10] * x10;
         y7 += A7[i+10] * x10;
         y0 += A0[i+11] * x11;
         y1 += A1[i+11] * x11;
         y2 += A2[i+11] * x11;
         y3 += A3[i+11] * x11;
         y4 += A4[i+11] * x11;
         y5 += A5[i+11] * x11;
         y6 += A6[i+11] * x11;
         y7 += A7[i+11] * x11;
         y0 += A0[i+12] * x12;
         y1 += A1[i+12] * x12;
         y2 += A2[i+12] * x12;
         y3 += A3[i+12] * x12;
         y4 += A4[i+12] * x12;
         y5 += A5[i+12] * x12;
         y6 += A6[i+12] * x12;
         y7 += A7[i+12] * x12;
         y0 += A0[i+13] * x13;
         y1 += A1[i+13] * x13;
         y2 += A2[i+13] * x13;
         y3 += A3[i+13] * x13;
         y4 += A4[i+13] * x13;
         y5 += A5[i+13] * x13;
         y6 += A6[i+13] * x13;
         y7 += A7[i+13] * x13;
         y0 += A0[i+14] * x14;
         y1 += A1[i+14] * x14;
         y2 += A2[i+14] * x14;
         y3 += A3[i+14] * x14;
         y4 += A4[i+14] * x14;
         y5 += A5[i+14] * x14;
         y6 += A6[i+14] * x14;
         y7 += A7[i+14] * x14;
         y0 += A0[i+15] * x15;
         y1 += A1[i+15] * x15;
         y2 += A2[i+15] * x15;
         y3 += A3[i+15] * x15;
         y4 += A4[i+15] * x15;
         y5 += A5[i+15] * x15;
         y6 += A6[i+15] * x15;
         y7 += A7[i+15] * x15;
         y0 += A0[i+16] * x16;
         prefA(A0);
         y1 += A1[i+16] * x16;
         prefA(A1);
         y2 += A2[i+16] * x16;
         prefA(A2);
         y3 += A3[i+16] * x16;
         prefA(A3);
         y4 += A4[i+16] * x16;
         prefA(A4);
         y5 += A5[i+16] * x16;
         prefA(A5);
         y6 += A6[i+16] * x16;
         prefA(A6);
         y7 += A7[i+16] * x16;
         prefA(A7);
         y0 += A0[i+17] * x17;
         y1 += A1[i+17] * x17;
         y2 += A2[i+17] * x17;
         y3 += A3[i+17] * x17;
         y4 += A4[i+17] * x17;
         y5 += A5[i+17] * x17;
         y6 += A6[i+17] * x17;
         y7 += A7[i+17] * x17;
         y0 += A0[i+18] * x18;
         y1 += A1[i+18] * x18;
         y2 += A2[i+18] * x18;
         y3 += A3[i+18] * x18;
         y4 += A4[i+18] * x18;
         y5 += A5[i+18] * x18;
         y6 += A6[i+18] * x18;
         y7 += A7[i+18] * x18;
         y0 += A0[i+19] * x19;
         y1 += A1[i+19] * x19;
         y2 += A2[i+19] * x19;
         y3 += A3[i+19] * x19;
         y4 += A4[i+19] * x19;
         y5 += A5[i+19] * x19;
         y6 += A6[i+19] * x19;
         y7 += A7[i+19] * x19;
         y0 += A0[i+20] * x20;
         y1 += A1[i+20] * x20;
         y2 += A2[i+20] * x20;
         y3 += A3[i+20] * x20;
         y4 += A4[i+20] * x20;
         y5 += A5[i+20] * x20;
         y6 += A6[i+20] * x20;
         y7 += A7[i+20] * x20;
         y0 += A0[i+21] * x21;
         y1 += A1[i+21] * x21;
         y2 += A2[i+21] * x21;
         y3 += A3[i+21] * x21;
         y4 += A4[i+21] * x21;
         y5 += A5[i+21] * x21;
         y6 += A6[i+21] * x21;
         y7 += A7[i+21] * x21;
         y0 += A0[i+22] * x22;
         y1 += A1[i+22] * x22;
         y2 += A2[i+22] * x22;
         y3 += A3[i+22] * x22;
         y4 += A4[i+22] * x22;
         y5 += A5[i+22] * x22;
         y6 += A6[i+22] * x22;
         y7 += A7[i+22] * x22;
         y0 += A0[i+23] * x23;
         y1 += A1[i+23] * x23;
         y2 += A2[i+23] * x23;
         y3 += A3[i+23] * x23;
         y4 += A4[i+23] * x23;
         y5 += A5[i+23] * x23;
         y6 += A6[i+23] * x23;
         y7 += A7[i+23] * x23;
         y0 += A0[i+24] * x24;
         y1 += A1[i+24] * x24;
         y2 += A2[i+24] * x24;
         y3 += A3[i+24] * x24;
         y4 += A4[i+24] * x24;
         y5 += A5[i+24] * x24;
         y6 += A6[i+24] * x24;
         y7 += A7[i+24] * x24;
         y0 += A0[i+25] * x25;
         y1 += A1[i+25] * x25;
         y2 += A2[i+25] * x25;
         y3 += A3[i+25] * x25;
         y4 += A4[i+25] * x25;
         y5 += A5[i+25] * x25;
         y6 += A6[i+25] * x25;
         y7 += A7[i+25] * x25;
         y0 += A0[i+26] * x26;
         y1 += A1[i+26] * x26;
         y2 += A2[i+26] * x26;
         y3 += A3[i+26] * x26;
         y4 += A4[i+26] * x26;
         y5 += A5[i+26] * x26;
         y6 += A6[i+26] * x26;
         y7 += A7[i+26] * x26;
         y0 += A0[i+27] * x27;
         y1 += A1[i+27] * x27;
         y2 += A2[i+27] * x27;
         y3 += A3[i+27] * x27;
         y4 += A4[i+27] * x27;
         y5 += A5[i+27] * x27;
         y6 += A6[i+27] * x27;
         y7 += A7[i+27] * x27;
         y0 += A0[i+28] * x28;
         y1 += A1[i+28] * x28;
         y2 += A2[i+28] * x28;
         y3 += A3[i+28] * x28;
         y4 += A4[i+28] * x28;
         y5 += A5[i+28] * x28;
         y6 += A6[i+28] * x28;
         y7 += A7[i+28] * x28;
         y0 += A0[i+29] * x29;
         y1 += A1[i+29] * x29;
         y2 += A2[i+29] * x29;
         y3 += A3[i+29] * x29;
         y4 += A4[i+29] * x29;
         y5 += A5[i+29] * x29;
         y6 += A6[i+29] * x29;
         y7 += A7[i+29] * x29;
         y0 += A0[i+30] * x30;
         y1 += A1[i+30] * x30;
         y2 += A2[i+30] * x30;
         y3 += A3[i+30] * x30;
         y4 += A4[i+30] * x30;
         y5 += A5[i+30] * x30;
         y6 += A6[i+30] * x30;
         y7 += A7[i+30] * x30;
         y0 += A0[i+31] * x31;
         y1 += A1[i+31] * x31;
         y2 += A2[i+31] * x31;
         y3 += A3[i+31] * x31;
         y4 += A4[i+31] * x31;
         y5 += A5[i+31] * x31;
         y6 += A6[i+31] * x31;
         y7 += A7[i+31] * x31;
         y0 += A0[i+32] * x32;
         prefA(A0);
         y1 += A1[i+32] * x32;
         prefA(A1);
         y2 += A2[i+32] * x32;
         prefA(A2);
         y3 += A3[i+32] * x32;
         prefA(A3);
         y4 += A4[i+32] * x32;
         prefA(A4);
         y5 += A5[i+32] * x32;
         prefA(A5);
         y6 += A6[i+32] * x32;
         prefA(A6);
         y7 += A7[i+32] * x32;
         prefA(A7);
         y0 += A0[i+33] * x33;
         y1 += A1[i+33] * x33;
         y2 += A2[i+33] * x33;
         y3 += A3[i+33] * x33;
         y4 += A4[i+33] * x33;
         y5 += A5[i+33] * x33;
         y6 += A6[i+33] * x33;
         y7 += A7[i+33] * x33;
         y0 += A0[i+34] * x34;
         y1 += A1[i+34] * x34;
         y2 += A2[i+34] * x34;
         y3 += A3[i+34] * x34;
         y4 += A4[i+34] * x34;
         y5 += A5[i+34] * x34;
         y6 += A6[i+34] * x34;
         y7 += A7[i+34] * x34;
         y0 += A0[i+35] * x35;
         y1 += A1[i+35] * x35;
         y2 += A2[i+35] * x35;
         y3 += A3[i+35] * x35;
         y4 += A4[i+35] * x35;
         y5 += A5[i+35] * x35;
         y6 += A6[i+35] * x35;
         y7 += A7[i+35] * x35;
         y0 += A0[i+36] * x36;
         y1 += A1[i+36] * x36;
         y2 += A2[i+36] * x36;
         y3 += A3[i+36] * x36;
         y4 += A4[i+36] * x36;
         y5 += A5[i+36] * x36;
         y6 += A6[i+36] * x36;
         y7 += A7[i+36] * x36;
         y0 += A0[i+37] * x37;
         y1 += A1[i+37] * x37;
         y2 += A2[i+37] * x37;
         y3 += A3[i+37] * x37;
         y4 += A4[i+37] * x37;
         y5 += A5[i+37] * x37;
         y6 += A6[i+37] * x37;
         y7 += A7[i+37] * x37;
         y0 += A0[i+38] * x38;
         y1 += A1[i+38] * x38;
         y2 += A2[i+38] * x38;
         y3 += A3[i+38] * x38;
         y4 += A4[i+38] * x38;
         y5 += A5[i+38] * x38;
         y6 += A6[i+38] * x38;
         y7 += A7[i+38] * x38;
         y0 += A0[i+39] * x39;
         y1 += A1[i+39] * x39;
         y2 += A2[i+39] * x39;
         y3 += A3[i+39] * x39;
         y4 += A4[i+39] * x39;
         y5 += A5[i+39] * x39;
         y6 += A6[i+39] * x39;
         y7 += A7[i+39] * x39;
         y0 += A0[i+40] * x40;
         y1 += A1[i+40] * x40;
         y2 += A2[i+40] * x40;
         y3 += A3[i+40] * x40;
         y4 += A4[i+40] * x40;
         y5 += A5[i+40] * x40;
         y6 += A6[i+40] * x40;
         y7 += A7[i+40] * x40;
         y0 += A0[i+41] * x41;
         y1 += A1[i+41] * x41;
         y2 += A2[i+41] * x41;
         y3 += A3[i+41] * x41;
         y4 += A4[i+41] * x41;
         y5 += A5[i+41] * x41;
         y6 += A6[i+41] * x41;
         y7 += A7[i+41] * x41;
         y0 += A0[i+42] * x42;
         y1 += A1[i+42] * x42;
         y2 += A2[i+42] * x42;
         y3 += A3[i+42] * x42;
         y4 += A4[i+42] * x42;
         y5 += A5[i+42] * x42;
         y6 += A6[i+42] * x42;
         y7 += A7[i+42] * x42;
         y0 += A0[i+43] * x43;
         y1 += A1[i+43] * x43;
         y2 += A2[i+43] * x43;
         y3 += A3[i+43] * x43;
         y4 += A4[i+43] * x43;
         y5 += A5[i+43] * x43;
         y6 += A6[i+43] * x43;
         y7 += A7[i+43] * x43;
         y0 += A0[i+44] * x44;
         y1 += A1[i+44] * x44;
         y2 += A2[i+44] * x44;
         y3 += A3[i+44] * x44;
         y4 += A4[i+44] * x44;
         y5 += A5[i+44] * x44;
         y6 += A6[i+44] * x44;
         y7 += A7[i+44] * x44;
         y0 += A0[i+45] * x45;
         y1 += A1[i+45] * x45;
         y2 += A2[i+45] * x45;
         y3 += A3[i+45] * x45;
         y4 += A4[i+45] * x45;
         y5 += A5[i+45] * x45;
         y6 += A6[i+45] * x45;
         y7 += A7[i+45] * x45;
         y0 += A0[i+46] * x46;
         y1 += A1[i+46] * x46;
         y2 += A2[i+46] * x46;
         y3 += A3[i+46] * x46;
         y4 += A4[i+46] * x46;
         y5 += A5[i+46] * x46;
         y6 += A6[i+46] * x46;
         y7 += A7[i+46] * x46;
         y0 += A0[i+47] * x47;
         y1 += A1[i+47] * x47;
         y2 += A2[i+47] * x47;
         y3 += A3[i+47] * x47;
         y4 += A4[i+47] * x47;
         y5 += A5[i+47] * x47;
         y6 += A6[i+47] * x47;
         y7 += A7[i+47] * x47;
         y0 += A0[i+48] * x48;
         prefA(A0);
         y1 += A1[i+48] * x48;
         prefA(A1);
         y2 += A2[i+48] * x48;
         prefA(A2);
         y3 += A3[i+48] * x48;
         prefA(A3);
         y4 += A4[i+48] * x48;
         prefA(A4);
         y5 += A5[i+48] * x48;
         prefA(A5);
         y6 += A6[i+48] * x48;
         prefA(A6);
         y7 += A7[i+48] * x48;
         prefA(A7);
         y0 += A0[i+49] * x49;
         y1 += A1[i+49] * x49;
         y2 += A2[i+49] * x49;
         y3 += A3[i+49] * x49;
         y4 += A4[i+49] * x49;
         y5 += A5[i+49] * x49;
         y6 += A6[i+49] * x49;
         y7 += A7[i+49] * x49;
         y0 += A0[i+50] * x50;
         y1 += A1[i+50] * x50;
         y2 += A2[i+50] * x50;
         y3 += A3[i+50] * x50;
         y4 += A4[i+50] * x50;
         y5 += A5[i+50] * x50;
         y6 += A6[i+50] * x50;
         y7 += A7[i+50] * x50;
         y0 += A0[i+51] * x51;
         y1 += A1[i+51] * x51;
         y2 += A2[i+51] * x51;
         y3 += A3[i+51] * x51;
         y4 += A4[i+51] * x51;
         y5 += A5[i+51] * x51;
         y6 += A6[i+51] * x51;
         y7 += A7[i+51] * x51;
         y0 += A0[i+52] * x52;
         y1 += A1[i+52] * x52;
         y2 += A2[i+52] * x52;
         y3 += A3[i+52] * x52;
         y4 += A4[i+52] * x52;
         y5 += A5[i+52] * x52;
         y6 += A6[i+52] * x52;
         y7 += A7[i+52] * x52;
         y0 += A0[i+53] * x53;
         y1 += A1[i+53] * x53;
         y2 += A2[i+53] * x53;
         y3 += A3[i+53] * x53;
         y4 += A4[i+53] * x53;
         y5 += A5[i+53] * x53;
         y6 += A6[i+53] * x53;
         y7 += A7[i+53] * x53;
         y0 += A0[i+54] * x54;
         y1 += A1[i+54] * x54;
         y2 += A2[i+54] * x54;
         y3 += A3[i+54] * x54;
         y4 += A4[i+54] * x54;
         y5 += A5[i+54] * x54;
         y6 += A6[i+54] * x54;
         y7 += A7[i+54] * x54;
         y0 += A0[i+55] * x55;
         y1 += A1[i+55] * x55;
         y2 += A2[i+55] * x55;
         y3 += A3[i+55] * x55;
         y4 += A4[i+55] * x55;
         y5 += A5[i+55] * x55;
         y6 += A6[i+55] * x55;
         y7 += A7[i+55] * x55;
         y0 += A0[i+56] * x56;
         y1 += A1[i+56] * x56;
         y2 += A2[i+56] * x56;
         y3 += A3[i+56] * x56;
         y4 += A4[i+56] * x56;
         y5 += A5[i+56] * x56;
         y6 += A6[i+56] * x56;
         y7 += A7[i+56] * x56;
         y0 += A0[i+57] * x57;
         y1 += A1[i+57] * x57;
         y2 += A2[i+57] * x57;
         y3 += A3[i+57] * x57;
         y4 += A4[i+57] * x57;
         y5 += A5[i+57] * x57;
         y6 += A6[i+57] * x57;
         y7 += A7[i+57] * x57;
         y0 += A0[i+58] * x58;
         y1 += A1[i+58] * x58;
         y2 += A2[i+58] * x58;
         y3 += A3[i+58] * x58;
         y4 += A4[i+58] * x58;
         y5 += A5[i+58] * x58;
         y6 += A6[i+58] * x58;
         y7 += A7[i+58] * x58;
         y0 += A0[i+59] * x59;
         y1 += A1[i+59] * x59;
         y2 += A2[i+59] * x59;
         y3 += A3[i+59] * x59;
         y4 += A4[i+59] * x59;
         y5 += A5[i+59] * x59;
         y6 += A6[i+59] * x59;
         y7 += A7[i+59] * x59;
         y0 += A0[i+60] * x60;
         y1 += A1[i+60] * x60;
         y2 += A2[i+60] * x60;
         y3 += A3[i+60] * x60;
         y4 += A4[i+60] * x60;
         y5 += A5[i+60] * x60;
         y6 += A6[i+60] * x60;
         y7 += A7[i+60] * x60;
         y0 += A0[i+61] * x61;
         y1 += A1[i+61] * x61;
         y2 += A2[i+61] * x61;
         y3 += A3[i+61] * x61;
         y4 += A4[i+61] * x61;
         y5 += A5[i+61] * x61;
         y6 += A6[i+61] * x61;
         y7 += A7[i+61] * x61;
         y0 += A0[i+62] * x62;
         y1 += A1[i+62] * x62;
         y2 += A2[i+62] * x62;
         y3 += A3[i+62] * x62;
         y4 += A4[i+62] * x62;
         y5 += A5[i+62] * x62;
         y6 += A6[i+62] * x62;
         y7 += A7[i+62] * x62;
         y0 += A0[i+63] * x63;
         y1 += A1[i+63] * x63;
         y2 += A2[i+63] * x63;
         y3 += A3[i+63] * x63;
         y4 += A4[i+63] * x63;
         y5 += A5[i+63] * x63;
         y6 += A6[i+63] * x63;
         y7 += A7[i+63] * x63;
         y0 += A0[i+64] * x64;
         prefA(A0);
         y1 += A1[i+64] * x64;
         prefA(A1);
         y2 += A2[i+64] * x64;
         prefA(A2);
         y3 += A3[i+64] * x64;
         prefA(A3);
         y4 += A4[i+64] * x64;
         prefA(A4);
         y5 += A5[i+64] * x64;
         prefA(A5);
         y6 += A6[i+64] * x64;
         prefA(A6);
         y7 += A7[i+64] * x64;
         prefA(A7);
         y0 += A0[i+65] * x65;
         y1 += A1[i+65] * x65;
         y2 += A2[i+65] * x65;
         y3 += A3[i+65] * x65;
         y4 += A4[i+65] * x65;
         y5 += A5[i+65] * x65;
         y6 += A6[i+65] * x65;
         y7 += A7[i+65] * x65;
         y0 += A0[i+66] * x66;
         y1 += A1[i+66] * x66;
         y2 += A2[i+66] * x66;
         y3 += A3[i+66] * x66;
         y4 += A4[i+66] * x66;
         y5 += A5[i+66] * x66;
         y6 += A6[i+66] * x66;
         y7 += A7[i+66] * x66;
         y0 += A0[i+67] * x67;
         y1 += A1[i+67] * x67;
         y2 += A2[i+67] * x67;
         y3 += A3[i+67] * x67;
         y4 += A4[i+67] * x67;
         y5 += A5[i+67] * x67;
         y6 += A6[i+67] * x67;
         y7 += A7[i+67] * x67;
         y0 += A0[i+68] * x68;
         y1 += A1[i+68] * x68;
         y2 += A2[i+68] * x68;
         y3 += A3[i+68] * x68;
         y4 += A4[i+68] * x68;
         y5 += A5[i+68] * x68;
         y6 += A6[i+68] * x68;
         y7 += A7[i+68] * x68;
         y0 += A0[i+69] * x69;
         y1 += A1[i+69] * x69;
         y2 += A2[i+69] * x69;
         y3 += A3[i+69] * x69;
         y4 += A4[i+69] * x69;
         y5 += A5[i+69] * x69;
         y6 += A6[i+69] * x69;
         y7 += A7[i+69] * x69;
         y0 += A0[i+70] * x70;
         y1 += A1[i+70] * x70;
         y2 += A2[i+70] * x70;
         y3 += A3[i+70] * x70;
         y4 += A4[i+70] * x70;
         y5 += A5[i+70] * x70;
         y6 += A6[i+70] * x70;
         y7 += A7[i+70] * x70;
         y0 += A0[i+71] * x71;
         y1 += A1[i+71] * x71;
         y2 += A2[i+71] * x71;
         y3 += A3[i+71] * x71;
         y4 += A4[i+71] * x71;
         y5 += A5[i+71] * x71;
         y6 += A6[i+71] * x71;
         y7 += A7[i+71] * x71;
         y0 += A0[i+72] * x72;
         y1 += A1[i+72] * x72;
         y2 += A2[i+72] * x72;
         y3 += A3[i+72] * x72;
         y4 += A4[i+72] * x72;
         y5 += A5[i+72] * x72;
         y6 += A6[i+72] * x72;
         y7 += A7[i+72] * x72;
         y0 += A0[i+73] * x73;
         y1 += A1[i+73] * x73;
         y2 += A2[i+73] * x73;
         y3 += A3[i+73] * x73;
         y4 += A4[i+73] * x73;
         y5 += A5[i+73] * x73;
         y6 += A6[i+73] * x73;
         y7 += A7[i+73] * x73;
         y0 += A0[i+74] * x74;
         y1 += A1[i+74] * x74;
         y2 += A2[i+74] * x74;
         y3 += A3[i+74] * x74;
         y4 += A4[i+74] * x74;
         y5 += A5[i+74] * x74;
         y6 += A6[i+74] * x74;
         y7 += A7[i+74] * x74;
         y0 += A0[i+75] * x75;
         y1 += A1[i+75] * x75;
         y2 += A2[i+75] * x75;
         y3 += A3[i+75] * x75;
         y4 += A4[i+75] * x75;
         y5 += A5[i+75] * x75;
         y6 += A6[i+75] * x75;
         y7 += A7[i+75] * x75;
         y0 += A0[i+76] * x76;
         y1 += A1[i+76] * x76;
         y2 += A2[i+76] * x76;
         y3 += A3[i+76] * x76;
         y4 += A4[i+76] * x76;
         y5 += A5[i+76] * x76;
         y6 += A6[i+76] * x76;
         y7 += A7[i+76] * x76;
         y0 += A0[i+77] * x77;
         y1 += A1[i+77] * x77;
         y2 += A2[i+77] * x77;
         y3 += A3[i+77] * x77;
         y4 += A4[i+77] * x77;
         y5 += A5[i+77] * x77;
         y6 += A6[i+77] * x77;
         y7 += A7[i+77] * x77;
         y0 += A0[i+78] * x78;
         y1 += A1[i+78] * x78;
         y2 += A2[i+78] * x78;
         y3 += A3[i+78] * x78;
         y4 += A4[i+78] * x78;
         y5 += A5[i+78] * x78;
         y6 += A6[i+78] * x78;
         y7 += A7[i+78] * x78;
         y0 += A0[i+79] * x79;
         y1 += A1[i+79] * x79;
         y2 += A2[i+79] * x79;
         y3 += A3[i+79] * x79;
         y4 += A4[i+79] * x79;
         y5 += A5[i+79] * x79;
         y6 += A6[i+79] * x79;
         y7 += A7[i+79] * x79;
         y0 += A0[i+80] * x80;
         prefA(A0);
         y1 += A1[i+80] * x80;
         prefA(A1);
         y2 += A2[i+80] * x80;
         prefA(A2);
         y3 += A3[i+80] * x80;
         prefA(A3);
         y4 += A4[i+80] * x80;
         prefA(A4);
         y5 += A5[i+80] * x80;
         prefA(A5);
         y6 += A6[i+80] * x80;
         prefA(A6);
         y7 += A7[i+80] * x80;
         prefA(A7);
         y0 += A0[i+81] * x81;
         y1 += A1[i+81] * x81;
         y2 += A2[i+81] * x81;
         y3 += A3[i+81] * x81;
         y4 += A4[i+81] * x81;
         y5 += A5[i+81] * x81;
         y6 += A6[i+81] * x81;
         y7 += A7[i+81] * x81;
         y0 += A0[i+82] * x82;
         y1 += A1[i+82] * x82;
         y2 += A2[i+82] * x82;
         y3 += A3[i+82] * x82;
         y4 += A4[i+82] * x82;
         y5 += A5[i+82] * x82;
         y6 += A6[i+82] * x82;
         y7 += A7[i+82] * x82;
         y0 += A0[i+83] * x83;
         y1 += A1[i+83] * x83;
         y2 += A2[i+83] * x83;
         y3 += A3[i+83] * x83;
         y4 += A4[i+83] * x83;
         y5 += A5[i+83] * x83;
         y6 += A6[i+83] * x83;
         y7 += A7[i+83] * x83;
         y0 += A0[i+84] * x84;
         y1 += A1[i+84] * x84;
         y2 += A2[i+84] * x84;
         y3 += A3[i+84] * x84;
         y4 += A4[i+84] * x84;
         y5 += A5[i+84] * x84;
         y6 += A6[i+84] * x84;
         y7 += A7[i+84] * x84;
         y0 += A0[i+85] * x85;
         y1 += A1[i+85] * x85;
         y2 += A2[i+85] * x85;
         y3 += A3[i+85] * x85;
         y4 += A4[i+85] * x85;
         y5 += A5[i+85] * x85;
         y6 += A6[i+85] * x85;
         y7 += A7[i+85] * x85;
         y0 += A0[i+86] * x86;
         y1 += A1[i+86] * x86;
         y2 += A2[i+86] * x86;
         y3 += A3[i+86] * x86;
         y4 += A4[i+86] * x86;
         y5 += A5[i+86] * x86;
         y6 += A6[i+86] * x86;
         y7 += A7[i+86] * x86;
         y0 += A0[i+87] * x87;
         y1 += A1[i+87] * x87;
         y2 += A2[i+87] * x87;
         y3 += A3[i+87] * x87;
         y4 += A4[i+87] * x87;
         y5 += A5[i+87] * x87;
         y6 += A6[i+87] * x87;
         y7 += A7[i+87] * x87;
         y0 += A0[i+88] * x88;
         y1 += A1[i+88] * x88;
         y2 += A2[i+88] * x88;
         y3 += A3[i+88] * x88;
         y4 += A4[i+88] * x88;
         y5 += A5[i+88] * x88;
         y6 += A6[i+88] * x88;
         y7 += A7[i+88] * x88;
         y0 += A0[i+89] * x89;
         y1 += A1[i+89] * x89;
         y2 += A2[i+89] * x89;
         y3 += A3[i+89] * x89;
         y4 += A4[i+89] * x89;
         y5 += A5[i+89] * x89;
         y6 += A6[i+89] * x89;
         y7 += A7[i+89] * x89;
         y0 += A0[i+90] * x90;
         y1 += A1[i+90] * x90;
         y2 += A2[i+90] * x90;
         y3 += A3[i+90] * x90;
         y4 += A4[i+90] * x90;
         y5 += A5[i+90] * x90;
         y6 += A6[i+90] * x90;
         y7 += A7[i+90] * x90;
         y0 += A0[i+91] * x91;
         y1 += A1[i+91] * x91;
         y2 += A2[i+91] * x91;
         y3 += A3[i+91] * x91;
         y4 += A4[i+91] * x91;
         y5 += A5[i+91] * x91;
         y6 += A6[i+91] * x91;
         y7 += A7[i+91] * x91;
         y0 += A0[i+92] * x92;
         y1 += A1[i+92] * x92;
         y2 += A2[i+92] * x92;
         y3 += A3[i+92] * x92;
         y4 += A4[i+92] * x92;
         y5 += A5[i+92] * x92;
         y6 += A6[i+92] * x92;
         y7 += A7[i+92] * x92;
         y0 += A0[i+93] * x93;
         y1 += A1[i+93] * x93;
         y2 += A2[i+93] * x93;
         y3 += A3[i+93] * x93;
         y4 += A4[i+93] * x93;
         y5 += A5[i+93] * x93;
         y6 += A6[i+93] * x93;
         y7 += A7[i+93] * x93;
         y0 += A0[i+94] * x94;
         y1 += A1[i+94] * x94;
         y2 += A2[i+94] * x94;
         y3 += A3[i+94] * x94;
         y4 += A4[i+94] * x94;
         y5 += A5[i+94] * x94;
         y6 += A6[i+94] * x94;
         y7 += A7[i+94] * x94;
         y0 += A0[i+95] * x95;
         y1 += A1[i+95] * x95;
         y2 += A2[i+95] * x95;
         y3 += A3[i+95] * x95;
         y4 += A4[i+95] * x95;
         y5 += A5[i+95] * x95;
         y6 += A6[i+95] * x95;
         y7 += A7[i+95] * x95;
         y0 += A0[i+96] * x96;
         prefA(A0);
         y1 += A1[i+96] * x96;
         prefA(A1);
         y2 += A2[i+96] * x96;
         prefA(A2);
         y3 += A3[i+96] * x96;
         prefA(A3);
         y4 += A4[i+96] * x96;
         prefA(A4);
         y5 += A5[i+96] * x96;
         prefA(A5);
         y6 += A6[i+96] * x96;
         prefA(A6);
         y7 += A7[i+96] * x96;
         prefA(A7);
         y0 += A0[i+97] * x97;
         y1 += A1[i+97] * x97;
         y2 += A2[i+97] * x97;
         y3 += A3[i+97] * x97;
         y4 += A4[i+97] * x97;
         y5 += A5[i+97] * x97;
         y6 += A6[i+97] * x97;
         y7 += A7[i+97] * x97;
         y0 += A0[i+98] * x98;
         y1 += A1[i+98] * x98;
         y2 += A2[i+98] * x98;
         y3 += A3[i+98] * x98;
         y4 += A4[i+98] * x98;
         y5 += A5[i+98] * x98;
         y6 += A6[i+98] * x98;
         y7 += A7[i+98] * x98;
         y0 += A0[i+99] * x99;
         y1 += A1[i+99] * x99;
         y2 += A2[i+99] * x99;
         y3 += A3[i+99] * x99;
         y4 += A4[i+99] * x99;
         y5 += A5[i+99] * x99;
         y6 += A6[i+99] * x99;
         y7 += A7[i+99] * x99;
         y0 += A0[i+100] * x100;
         y1 += A1[i+100] * x100;
         y2 += A2[i+100] * x100;
         y3 += A3[i+100] * x100;
         y4 += A4[i+100] * x100;
         y5 += A5[i+100] * x100;
         y6 += A6[i+100] * x100;
         y7 += A7[i+100] * x100;
         y0 += A0[i+101] * x101;
         y1 += A1[i+101] * x101;
         y2 += A2[i+101] * x101;
         y3 += A3[i+101] * x101;
         y4 += A4[i+101] * x101;
         y5 += A5[i+101] * x101;
         y6 += A6[i+101] * x101;
         y7 += A7[i+101] * x101;
         y0 += A0[i+102] * x102;
         y1 += A1[i+102] * x102;
         y2 += A2[i+102] * x102;
         y3 += A3[i+102] * x102;
         y4 += A4[i+102] * x102;
         y5 += A5[i+102] * x102;
         y6 += A6[i+102] * x102;
         y7 += A7[i+102] * x102;
         y0 += A0[i+103] * x103;
         y1 += A1[i+103] * x103;
         y2 += A2[i+103] * x103;
         y3 += A3[i+103] * x103;
         y4 += A4[i+103] * x103;
         y5 += A5[i+103] * x103;
         y6 += A6[i+103] * x103;
         y7 += A7[i+103] * x103;
         y0 += A0[i+104] * x104;
         y1 += A1[i+104] * x104;
         y2 += A2[i+104] * x104;
         y3 += A3[i+104] * x104;
         y4 += A4[i+104] * x104;
         y5 += A5[i+104] * x104;
         y6 += A6[i+104] * x104;
         y7 += A7[i+104] * x104;
         y0 += A0[i+105] * x105;
         y1 += A1[i+105] * x105;
         y2 += A2[i+105] * x105;
         y3 += A3[i+105] * x105;
         y4 += A4[i+105] * x105;
         y5 += A5[i+105] * x105;
         y6 += A6[i+105] * x105;
         y7 += A7[i+105] * x105;
         y0 += A0[i+106] * x106;
         y1 += A1[i+106] * x106;
         y2 += A2[i+106] * x106;
         y3 += A3[i+106] * x106;
         y4 += A4[i+106] * x106;
         y5 += A5[i+106] * x106;
         y6 += A6[i+106] * x106;
         y7 += A7[i+106] * x106;
         y0 += A0[i+107] * x107;
         y1 += A1[i+107] * x107;
         y2 += A2[i+107] * x107;
         y3 += A3[i+107] * x107;
         y4 += A4[i+107] * x107;
         y5 += A5[i+107] * x107;
         y6 += A6[i+107] * x107;
         y7 += A7[i+107] * x107;
         y0 += A0[i+108] * x108;
         y1 += A1[i+108] * x108;
         y2 += A2[i+108] * x108;
         y3 += A3[i+108] * x108;
         y4 += A4[i+108] * x108;
         y5 += A5[i+108] * x108;
         y6 += A6[i+108] * x108;
         y7 += A7[i+108] * x108;
         y0 += A0[i+109] * x109;
         y1 += A1[i+109] * x109;
         y2 += A2[i+109] * x109;
         y3 += A3[i+109] * x109;
         y4 += A4[i+109] * x109;
         y5 += A5[i+109] * x109;
         y6 += A6[i+109] * x109;
         y7 += A7[i+109] * x109;
         y0 += A0[i+110] * x110;
         y1 += A1[i+110] * x110;
         y2 += A2[i+110] * x110;
         y3 += A3[i+110] * x110;
         y4 += A4[i+110] * x110;
         y5 += A5[i+110] * x110;
         y6 += A6[i+110] * x110;
         y7 += A7[i+110] * x110;
         y0 += A0[i+111] * x111;
         y1 += A1[i+111] * x111;
         y2 += A2[i+111] * x111;
         y3 += A3[i+111] * x111;
         y4 += A4[i+111] * x111;
         y5 += A5[i+111] * x111;
         y6 += A6[i+111] * x111;
         y7 += A7[i+111] * x111;
         y0 += A0[i+112] * x112;
         prefA(A0);
         y1 += A1[i+112] * x112;
         prefA(A1);
         y2 += A2[i+112] * x112;
         prefA(A2);
         y3 += A3[i+112] * x112;
         prefA(A3);
         y4 += A4[i+112] * x112;
         prefA(A4);
         y5 += A5[i+112] * x112;
         prefA(A5);
         y6 += A6[i+112] * x112;
         prefA(A6);
         y7 += A7[i+112] * x112;
         prefA(A7);
         y0 += A0[i+113] * x113;
         y1 += A1[i+113] * x113;
         y2 += A2[i+113] * x113;
         y3 += A3[i+113] * x113;
         y4 += A4[i+113] * x113;
         y5 += A5[i+113] * x113;
         y6 += A6[i+113] * x113;
         y7 += A7[i+113] * x113;
         y0 += A0[i+114] * x114;
         y1 += A1[i+114] * x114;
         y2 += A2[i+114] * x114;
         y3 += A3[i+114] * x114;
         y4 += A4[i+114] * x114;
         y5 += A5[i+114] * x114;
         y6 += A6[i+114] * x114;
         y7 += A7[i+114] * x114;
         y0 += A0[i+115] * x115;
         y1 += A1[i+115] * x115;
         y2 += A2[i+115] * x115;
         y3 += A3[i+115] * x115;
         y4 += A4[i+115] * x115;
         y5 += A5[i+115] * x115;
         y6 += A6[i+115] * x115;
         y7 += A7[i+115] * x115;
         y0 += A0[i+116] * x116;
         y1 += A1[i+116] * x116;
         y2 += A2[i+116] * x116;
         y3 += A3[i+116] * x116;
         y4 += A4[i+116] * x116;
         y5 += A5[i+116] * x116;
         y6 += A6[i+116] * x116;
         y7 += A7[i+116] * x116;
         y0 += A0[i+117] * x117;
         y1 += A1[i+117] * x117;
         y2 += A2[i+117] * x117;
         y3 += A3[i+117] * x117;
         y4 += A4[i+117] * x117;
         y5 += A5[i+117] * x117;
         y6 += A6[i+117] * x117;
         y7 += A7[i+117] * x117;
         y0 += A0[i+118] * x118;
         y1 += A1[i+118] * x118;
         y2 += A2[i+118] * x118;
         y3 += A3[i+118] * x118;
         y4 += A4[i+118] * x118;
         y5 += A5[i+118] * x118;
         y6 += A6[i+118] * x118;
         y7 += A7[i+118] * x118;
         y0 += A0[i+119] * x119;
         y1 += A1[i+119] * x119;
         y2 += A2[i+119] * x119;
         y3 += A3[i+119] * x119;
         y4 += A4[i+119] * x119;
         y5 += A5[i+119] * x119;
         y6 += A6[i+119] * x119;
         y7 += A7[i+119] * x119;
         y0 += A0[i+120] * x120;
         y1 += A1[i+120] * x120;
         y2 += A2[i+120] * x120;
         y3 += A3[i+120] * x120;
         y4 += A4[i+120] * x120;
         y5 += A5[i+120] * x120;
         y6 += A6[i+120] * x120;
         y7 += A7[i+120] * x120;
         y0 += A0[i+121] * x121;
         y1 += A1[i+121] * x121;
         y2 += A2[i+121] * x121;
         y3 += A3[i+121] * x121;
         y4 += A4[i+121] * x121;
         y5 += A5[i+121] * x121;
         y6 += A6[i+121] * x121;
         y7 += A7[i+121] * x121;
         y0 += A0[i+122] * x122;
         y1 += A1[i+122] * x122;
         y2 += A2[i+122] * x122;
         y3 += A3[i+122] * x122;
         y4 += A4[i+122] * x122;
         y5 += A5[i+122] * x122;
         y6 += A6[i+122] * x122;
         y7 += A7[i+122] * x122;
         y0 += A0[i+123] * x123;
         y1 += A1[i+123] * x123;
         y2 += A2[i+123] * x123;
         y3 += A3[i+123] * x123;
         y4 += A4[i+123] * x123;
         y5 += A5[i+123] * x123;
         y6 += A6[i+123] * x123;
         y7 += A7[i+123] * x123;
         y0 += A0[i+124] * x124;
         y1 += A1[i+124] * x124;
         y2 += A2[i+124] * x124;
         y3 += A3[i+124] * x124;
         y4 += A4[i+124] * x124;
         y5 += A5[i+124] * x124;
         y6 += A6[i+124] * x124;
         y7 += A7[i+124] * x124;
         y0 += A0[i+125] * x125;
         y1 += A1[i+125] * x125;
         y2 += A2[i+125] * x125;
         y3 += A3[i+125] * x125;
         y4 += A4[i+125] * x125;
         y5 += A5[i+125] * x125;
         y6 += A6[i+125] * x125;
         y7 += A7[i+125] * x125;
         y0 += A0[i+126] * x126;
         y1 += A1[i+126] * x126;
         y2 += A2[i+126] * x126;
         y3 += A3[i+126] * x126;
         y4 += A4[i+126] * x126;
         y5 += A5[i+126] * x126;
         y6 += A6[i+126] * x126;
         y7 += A7[i+126] * x126;
         y0 += A0[i+127] * x127;
         y1 += A1[i+127] * x127;
         y2 += A2[i+127] * x127;
         y3 += A3[i+127] * x127;
         y4 += A4[i+127] * x127;
         y5 += A5[i+127] * x127;
         y6 += A6[i+127] * x127;
         y7 += A7[i+127] * x127;
      }
      for (i=M128; i < M; i++)
      {
         const register float x0 = X[i];
         y0 += A0[i] * x0;
         y1 += A1[i] * x0;
         y2 += A2[i] * x0;
         y3 += A3[i] * x0;
         y4 += A4[i] * x0;
         y5 += A5[i] * x0;
         y6 += A6[i] * x0;
         y7 += A7[i] * x0;
      }
      #ifdef BETA0
         Y[0] = y0;
         Y[1] = y1;
         Y[2] = y2;
         Y[3] = y3;
         Y[4] = y4;
         Y[5] = y5;
         Y[6] = y6;
         Y[7] = y7;
      #else
         Y[0] += y0;
         Y[1] += y1;
         Y[2] += y2;
         Y[3] += y3;
         Y[4] += y4;
         Y[5] += y5;
         Y[6] += y6;
         Y[7] += y7;
      #endif
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N8; j; j--, A += lda, Y++)
   {
      const float *A0=A;
      register float y0=ATL_rzero;
      ATL_INT i;
      prefY(Y+1+1-1);
      for (i=0; i < M128; i += 128)
      {
         const register float x0=X[i+0], x1=X[i+1], x2=X[i+2], x3=X[i+3],
                              x4=X[i+4], x5=X[i+5], x6=X[i+6], x7=X[i+7],
                              x8=X[i+8], x9=X[i+9], x10=X[i+10], x11=X[i+11],
                              x12=X[i+12], x13=X[i+13], x14=X[i+14],
                              x15=X[i+15], x16=X[i+16], x17=X[i+17],
                              x18=X[i+18], x19=X[i+19], x20=X[i+20],
                              x21=X[i+21], x22=X[i+22], x23=X[i+23],
                              x24=X[i+24], x25=X[i+25], x26=X[i+26],
                              x27=X[i+27], x28=X[i+28], x29=X[i+29],
                              x30=X[i+30], x31=X[i+31], x32=X[i+32],
                              x33=X[i+33], x34=X[i+34], x35=X[i+35],
                              x36=X[i+36], x37=X[i+37], x38=X[i+38],
                              x39=X[i+39], x40=X[i+40], x41=X[i+41],
                              x42=X[i+42], x43=X[i+43], x44=X[i+44],
                              x45=X[i+45], x46=X[i+46], x47=X[i+47],
                              x48=X[i+48], x49=X[i+49], x50=X[i+50],
                              x51=X[i+51], x52=X[i+52], x53=X[i+53],
                              x54=X[i+54], x55=X[i+55], x56=X[i+56],
                              x57=X[i+57], x58=X[i+58], x59=X[i+59],
                              x60=X[i+60], x61=X[i+61], x62=X[i+62],
                              x63=X[i+63], x64=X[i+64], x65=X[i+65],
                              x66=X[i+66], x67=X[i+67], x68=X[i+68],
                              x69=X[i+69], x70=X[i+70], x71=X[i+71],
                              x72=X[i+72], x73=X[i+73], x74=X[i+74],
                              x75=X[i+75], x76=X[i+76], x77=X[i+77],
                              x78=X[i+78], x79=X[i+79], x80=X[i+80],
                              x81=X[i+81], x82=X[i+82], x83=X[i+83],
                              x84=X[i+84], x85=X[i+85], x86=X[i+86],
                              x87=X[i+87], x88=X[i+88], x89=X[i+89],
                              x90=X[i+90], x91=X[i+91], x92=X[i+92],
                              x93=X[i+93], x94=X[i+94], x95=X[i+95],
                              x96=X[i+96], x97=X[i+97], x98=X[i+98],
                              x99=X[i+99], x100=X[i+100], x101=X[i+101],
                              x102=X[i+102], x103=X[i+103], x104=X[i+104],
                              x105=X[i+105], x106=X[i+106], x107=X[i+107],
                              x108=X[i+108], x109=X[i+109], x110=X[i+110],
                              x111=X[i+111], x112=X[i+112], x113=X[i+113],
                              x114=X[i+114], x115=X[i+115], x116=X[i+116],
                              x117=X[i+117], x118=X[i+118], x119=X[i+119],
                              x120=X[i+120], x121=X[i+121], x122=X[i+122],
                              x123=X[i+123], x124=X[i+124], x125=X[i+125],
                              x126=X[i+126], x127=X[i+127];

         prefX(X);
         y0 += A0[i+0] * x0;
         prefA(A0);
         y0 += A0[i+1] * x1;
         y0 += A0[i+2] * x2;
         y0 += A0[i+3] * x3;
         y0 += A0[i+4] * x4;
         y0 += A0[i+5] * x5;
         y0 += A0[i+6] * x6;
         y0 += A0[i+7] * x7;
         y0 += A0[i+8] * x8;
         y0 += A0[i+9] * x9;
         y0 += A0[i+10] * x10;
         y0 += A0[i+11] * x11;
         y0 += A0[i+12] * x12;
         y0 += A0[i+13] * x13;
         y0 += A0[i+14] * x14;
         y0 += A0[i+15] * x15;
         y0 += A0[i+16] * x16;
         prefA(A0);
         y0 += A0[i+17] * x17;
         y0 += A0[i+18] * x18;
         y0 += A0[i+19] * x19;
         y0 += A0[i+20] * x20;
         y0 += A0[i+21] * x21;
         y0 += A0[i+22] * x22;
         y0 += A0[i+23] * x23;
         y0 += A0[i+24] * x24;
         y0 += A0[i+25] * x25;
         y0 += A0[i+26] * x26;
         y0 += A0[i+27] * x27;
         y0 += A0[i+28] * x28;
         y0 += A0[i+29] * x29;
         y0 += A0[i+30] * x30;
         y0 += A0[i+31] * x31;
         y0 += A0[i+32] * x32;
         prefA(A0);
         y0 += A0[i+33] * x33;
         y0 += A0[i+34] * x34;
         y0 += A0[i+35] * x35;
         y0 += A0[i+36] * x36;
         y0 += A0[i+37] * x37;
         y0 += A0[i+38] * x38;
         y0 += A0[i+39] * x39;
         y0 += A0[i+40] * x40;
         y0 += A0[i+41] * x41;
         y0 += A0[i+42] * x42;
         y0 += A0[i+43] * x43;
         y0 += A0[i+44] * x44;
         y0 += A0[i+45] * x45;
         y0 += A0[i+46] * x46;
         y0 += A0[i+47] * x47;
         y0 += A0[i+48] * x48;
         prefA(A0);
         y0 += A0[i+49] * x49;
         y0 += A0[i+50] * x50;
         y0 += A0[i+51] * x51;
         y0 += A0[i+52] * x52;
         y0 += A0[i+53] * x53;
         y0 += A0[i+54] * x54;
         y0 += A0[i+55] * x55;
         y0 += A0[i+56] * x56;
         y0 += A0[i+57] * x57;
         y0 += A0[i+58] * x58;
         y0 += A0[i+59] * x59;
         y0 += A0[i+60] * x60;
         y0 += A0[i+61] * x61;
         y0 += A0[i+62] * x62;
         y0 += A0[i+63] * x63;
         y0 += A0[i+64] * x64;
         prefA(A0);
         y0 += A0[i+65] * x65;
         y0 += A0[i+66] * x66;
         y0 += A0[i+67] * x67;
         y0 += A0[i+68] * x68;
         y0 += A0[i+69] * x69;
         y0 += A0[i+70] * x70;
         y0 += A0[i+71] * x71;
         y0 += A0[i+72] * x72;
         y0 += A0[i+73] * x73;
         y0 += A0[i+74] * x74;
         y0 += A0[i+75] * x75;
         y0 += A0[i+76] * x76;
         y0 += A0[i+77] * x77;
         y0 += A0[i+78] * x78;
         y0 += A0[i+79] * x79;
         y0 += A0[i+80] * x80;
         prefA(A0);
         y0 += A0[i+81] * x81;
         y0 += A0[i+82] * x82;
         y0 += A0[i+83] * x83;
         y0 += A0[i+84] * x84;
         y0 += A0[i+85] * x85;
         y0 += A0[i+86] * x86;
         y0 += A0[i+87] * x87;
         y0 += A0[i+88] * x88;
         y0 += A0[i+89] * x89;
         y0 += A0[i+90] * x90;
         y0 += A0[i+91] * x91;
         y0 += A0[i+92] * x92;
         y0 += A0[i+93] * x93;
         y0 += A0[i+94] * x94;
         y0 += A0[i+95] * x95;
         y0 += A0[i+96] * x96;
         prefA(A0);
         y0 += A0[i+97] * x97;
         y0 += A0[i+98] * x98;
         y0 += A0[i+99] * x99;
         y0 += A0[i+100] * x100;
         y0 += A0[i+101] * x101;
         y0 += A0[i+102] * x102;
         y0 += A0[i+103] * x103;
         y0 += A0[i+104] * x104;
         y0 += A0[i+105] * x105;
         y0 += A0[i+106] * x106;
         y0 += A0[i+107] * x107;
         y0 += A0[i+108] * x108;
         y0 += A0[i+109] * x109;
         y0 += A0[i+110] * x110;
         y0 += A0[i+111] * x111;
         y0 += A0[i+112] * x112;
         prefA(A0);
         y0 += A0[i+113] * x113;
         y0 += A0[i+114] * x114;
         y0 += A0[i+115] * x115;
         y0 += A0[i+116] * x116;
         y0 += A0[i+117] * x117;
         y0 += A0[i+118] * x118;
         y0 += A0[i+119] * x119;
         y0 += A0[i+120] * x120;
         y0 += A0[i+121] * x121;
         y0 += A0[i+122] * x122;
         y0 += A0[i+123] * x123;
         y0 += A0[i+124] * x124;
         y0 += A0[i+125] * x125;
         y0 += A0[i+126] * x126;
         y0 += A0[i+127] * x127;
      }
      for (i=M128; i < M; i++)
      {
         const register float x0 = X[i];
         y0 += A0[i] * x0;
      }
      #ifdef BETA0
         Y[0] = y0;
      #else
         Y[0] += y0;
      #endif
   }
}
